# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.Compute.V1.Model.Autoscaler do
  @moduledoc """
  Represents an Autoscaler resource. Google Compute Engine has two Autoscaler resources: * [Zonal](/compute/docs/reference/rest/v1/autoscalers) * [Regional](/compute/docs/reference/rest/v1/regionAutoscalers) Use autoscalers to automatically add or delete instances from a managed instance group according to your defined autoscaling policy. For more information, read Autoscaling Groups of Instances. For zonal managed instance groups resource, use the autoscaler resource. For regional managed instance groups, use the regionAutoscalers resource.

  ## Attributes

  *   `autoscalingPolicy` (*type:* `GoogleApi.Compute.V1.Model.AutoscalingPolicy.t`, *default:* `nil`) - The configuration parameters for the autoscaling algorithm. You can define one or more signals for an autoscaler: cpuUtilization, customMetricUtilizations, and loadBalancingUtilization. If none of these are specified, the default will be to autoscale based on cpuUtilization to 0.6 or 60%.
  *   `creationTimestamp` (*type:* `String.t`, *default:* `nil`) - [Output Only] Creation timestamp in RFC3339 text format.
  *   `description` (*type:* `String.t`, *default:* `nil`) - An optional description of this resource. Provide this property when you create the resource.
  *   `id` (*type:* `String.t`, *default:* `nil`) - [Output Only] The unique identifier for the resource. This identifier is defined by the server.
  *   `kind` (*type:* `String.t`, *default:* `compute#autoscaler`) - [Output Only] Type of the resource. Always compute#autoscaler for autoscalers.
  *   `name` (*type:* `String.t`, *default:* `nil`) - Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
  *   `recommendedSize` (*type:* `integer()`, *default:* `nil`) - [Output Only] Target recommended MIG size (number of instances) computed by autoscaler. Autoscaler calculates the recommended MIG size even when the autoscaling policy mode is different from ON. This field is empty when autoscaler is not connected to an existing managed instance group or autoscaler did not generate its prediction.
  *   `region` (*type:* `String.t`, *default:* `nil`) - [Output Only] URL of the region where the instance group resides (for autoscalers living in regional scope).
  *   `scalingScheduleStatus` (*type:* `%{optional(String.t) => GoogleApi.Compute.V1.Model.ScalingScheduleStatus.t}`, *default:* `nil`) - [Output Only] Status information of existing scaling schedules.
  *   `selfLink` (*type:* `String.t`, *default:* `nil`) - [Output Only] Server-defined URL for the resource.
  *   `status` (*type:* `String.t`, *default:* `nil`) - [Output Only] The status of the autoscaler configuration. Current set of possible values: - PENDING: Autoscaler backend hasn't read new/updated configuration. - DELETING: Configuration is being deleted. - ACTIVE: Configuration is acknowledged to be effective. Some warnings might be present in the statusDetails field. - ERROR: Configuration has errors. Actionable for users. Details are present in the statusDetails field. New values might be added in the future.
  *   `statusDetails` (*type:* `list(GoogleApi.Compute.V1.Model.AutoscalerStatusDetails.t)`, *default:* `nil`) - [Output Only] Human-readable details about the current state of the autoscaler. Read the documentation for Commonly returned status messages for examples of status messages you might encounter.
  *   `target` (*type:* `String.t`, *default:* `nil`) - URL of the managed instance group that this autoscaler will scale. This field is required when creating an autoscaler.
  *   `zone` (*type:* `String.t`, *default:* `nil`) - [Output Only] URL of the zone where the instance group resides (for autoscalers living in zonal scope).
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :autoscalingPolicy => GoogleApi.Compute.V1.Model.AutoscalingPolicy.t() | nil,
          :creationTimestamp => String.t() | nil,
          :description => String.t() | nil,
          :id => String.t() | nil,
          :kind => String.t() | nil,
          :name => String.t() | nil,
          :recommendedSize => integer() | nil,
          :region => String.t() | nil,
          :scalingScheduleStatus =>
            %{optional(String.t()) => GoogleApi.Compute.V1.Model.ScalingScheduleStatus.t()} | nil,
          :selfLink => String.t() | nil,
          :status => String.t() | nil,
          :statusDetails => list(GoogleApi.Compute.V1.Model.AutoscalerStatusDetails.t()) | nil,
          :target => String.t() | nil,
          :zone => String.t() | nil
        }

  field(:autoscalingPolicy, as: GoogleApi.Compute.V1.Model.AutoscalingPolicy)
  field(:creationTimestamp)
  field(:description)
  field(:id)
  field(:kind)
  field(:name)
  field(:recommendedSize)
  field(:region)
  field(:scalingScheduleStatus, as: GoogleApi.Compute.V1.Model.ScalingScheduleStatus, type: :map)
  field(:selfLink)
  field(:status)
  field(:statusDetails, as: GoogleApi.Compute.V1.Model.AutoscalerStatusDetails, type: :list)
  field(:target)
  field(:zone)
end

defimpl Poison.Decoder, for: GoogleApi.Compute.V1.Model.Autoscaler do
  def decode(value, options) do
    GoogleApi.Compute.V1.Model.Autoscaler.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.Compute.V1.Model.Autoscaler do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
