# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.Compute.V1.Model.FirewallPolicyRule do
  @moduledoc """
  Represents a rule that describes one or more match conditions along with the action to be taken when traffic matches this condition (allow or deny).

  ## Attributes

  *   `action` (*type:* `String.t`, *default:* `nil`) - The Action to perform when the client connection triggers the rule. Valid actions are "allow", "deny" and "goto_next".
  *   `description` (*type:* `String.t`, *default:* `nil`) - An optional description for this resource.
  *   `direction` (*type:* `String.t`, *default:* `nil`) - The direction in which this rule applies.
  *   `disabled` (*type:* `boolean()`, *default:* `nil`) - Denotes whether the firewall policy rule is disabled. When set to true, the firewall policy rule is not enforced and traffic behaves as if it did not exist. If this is unspecified, the firewall policy rule will be enabled.
  *   `enableLogging` (*type:* `boolean()`, *default:* `nil`) - Denotes whether to enable logging for a particular rule. If logging is enabled, logs will be exported to the configured export destination in Stackdriver. Logs may be exported to BigQuery or Pub/Sub. Note: you cannot enable logging on "goto_next" rules.
  *   `kind` (*type:* `String.t`, *default:* `compute#firewallPolicyRule`) - [Output only] Type of the resource. Always compute#firewallPolicyRule for firewall policy rules
  *   `match` (*type:* `GoogleApi.Compute.V1.Model.FirewallPolicyRuleMatcher.t`, *default:* `nil`) - A match condition that incoming traffic is evaluated against. If it evaluates to true, the corresponding 'action' is enforced.
  *   `priority` (*type:* `integer()`, *default:* `nil`) - An integer indicating the priority of a rule in the list. The priority must be a positive value between 0 and 2147483647. Rules are evaluated from highest to lowest priority where 0 is the highest priority and 2147483647 is the lowest prority.
  *   `ruleName` (*type:* `String.t`, *default:* `nil`) - An optional name for the rule. This field is not a unique identifier and can be updated.
  *   `ruleTupleCount` (*type:* `integer()`, *default:* `nil`) - [Output Only] Calculation of the complexity of a single firewall policy rule.
  *   `securityProfileGroup` (*type:* `String.t`, *default:* `nil`) - A fully-qualified URL of a SecurityProfile resource instance. Example: https://networksecurity.googleapis.com/v1/projects/{project}/locations/{location}/securityProfileGroups/my-security-profile-group Must be specified if action = 'apply_security_profile_group' and cannot be specified for other actions.
  *   `targetResources` (*type:* `list(String.t)`, *default:* `nil`) - A list of network resource URLs to which this rule applies. This field allows you to control which network's VMs get this rule. If this field is left blank, all VMs within the organization will receive the rule.
  *   `targetSecureTags` (*type:* `list(GoogleApi.Compute.V1.Model.FirewallPolicyRuleSecureTag.t)`, *default:* `nil`) - A list of secure tags that controls which instances the firewall rule applies to. If targetSecureTag are specified, then the firewall rule applies only to instances in the VPC network that have one of those EFFECTIVE secure tags, if all the target_secure_tag are in INEFFECTIVE state, then this rule will be ignored. targetSecureTag may not be set at the same time as targetServiceAccounts. If neither targetServiceAccounts nor targetSecureTag are specified, the firewall rule applies to all instances on the specified network. Maximum number of target label tags allowed is 256.
  *   `targetServiceAccounts` (*type:* `list(String.t)`, *default:* `nil`) - A list of service accounts indicating the sets of instances that are applied with this rule.
  *   `tlsInspect` (*type:* `boolean()`, *default:* `nil`) - Boolean flag indicating if the traffic should be TLS decrypted. Can be set only if action = 'apply_security_profile_group' and cannot be set for other actions.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :action => String.t() | nil,
          :description => String.t() | nil,
          :direction => String.t() | nil,
          :disabled => boolean() | nil,
          :enableLogging => boolean() | nil,
          :kind => String.t() | nil,
          :match => GoogleApi.Compute.V1.Model.FirewallPolicyRuleMatcher.t() | nil,
          :priority => integer() | nil,
          :ruleName => String.t() | nil,
          :ruleTupleCount => integer() | nil,
          :securityProfileGroup => String.t() | nil,
          :targetResources => list(String.t()) | nil,
          :targetSecureTags =>
            list(GoogleApi.Compute.V1.Model.FirewallPolicyRuleSecureTag.t()) | nil,
          :targetServiceAccounts => list(String.t()) | nil,
          :tlsInspect => boolean() | nil
        }

  field(:action)
  field(:description)
  field(:direction)
  field(:disabled)
  field(:enableLogging)
  field(:kind)
  field(:match, as: GoogleApi.Compute.V1.Model.FirewallPolicyRuleMatcher)
  field(:priority)
  field(:ruleName)
  field(:ruleTupleCount)
  field(:securityProfileGroup)
  field(:targetResources, type: :list)

  field(:targetSecureTags, as: GoogleApi.Compute.V1.Model.FirewallPolicyRuleSecureTag, type: :list)

  field(:targetServiceAccounts, type: :list)
  field(:tlsInspect)
end

defimpl Poison.Decoder, for: GoogleApi.Compute.V1.Model.FirewallPolicyRule do
  def decode(value, options) do
    GoogleApi.Compute.V1.Model.FirewallPolicyRule.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.Compute.V1.Model.FirewallPolicyRule do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
