# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.Compute.V1.Model.PublicDelegatedPrefix do
  @moduledoc """
  A PublicDelegatedPrefix resource represents an IP block within a PublicAdvertisedPrefix that is configured within a single cloud scope (global or region). IPs in the block can be allocated to resources within that scope. Public delegated prefixes may be further broken up into smaller IP blocks in the same scope as the parent block.

  ## Attributes

  *   `byoipApiVersion` (*type:* `String.t`, *default:* `nil`) - [Output Only] The version of BYOIP API.
  *   `creationTimestamp` (*type:* `String.t`, *default:* `nil`) - [Output Only] Creation timestamp in RFC3339 text format.
  *   `description` (*type:* `String.t`, *default:* `nil`) - An optional description of this resource. Provide this property when you create the resource.
  *   `fingerprint` (*type:* `String.t`, *default:* `nil`) - Fingerprint of this resource. A hash of the contents stored in this object. This field is used in optimistic locking. This field will be ignored when inserting a new PublicDelegatedPrefix. An up-to-date fingerprint must be provided in order to update the PublicDelegatedPrefix, otherwise the request will fail with error 412 conditionNotMet. To see the latest fingerprint, make a get() request to retrieve a PublicDelegatedPrefix.
  *   `id` (*type:* `String.t`, *default:* `nil`) - [Output Only] The unique identifier for the resource type. The server generates this identifier.
  *   `ipCidrRange` (*type:* `String.t`, *default:* `nil`) - The IP address range, in CIDR format, represented by this public delegated prefix.
  *   `isLiveMigration` (*type:* `boolean()`, *default:* `nil`) - If true, the prefix will be live migrated.
  *   `kind` (*type:* `String.t`, *default:* `compute#publicDelegatedPrefix`) - [Output Only] Type of the resource. Always compute#publicDelegatedPrefix for public delegated prefixes.
  *   `name` (*type:* `String.t`, *default:* `nil`) - Name of the resource. Provided by the client when the resource is created. The name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
  *   `parentPrefix` (*type:* `String.t`, *default:* `nil`) - The URL of parent prefix. Either PublicAdvertisedPrefix or PublicDelegatedPrefix.
  *   `publicDelegatedSubPrefixs` (*type:* `list(GoogleApi.Compute.V1.Model.PublicDelegatedPrefixPublicDelegatedSubPrefix.t)`, *default:* `nil`) - The list of sub public delegated prefixes that exist for this public delegated prefix.
  *   `region` (*type:* `String.t`, *default:* `nil`) - [Output Only] URL of the region where the public delegated prefix resides. This field applies only to the region resource. You must specify this field as part of the HTTP request URL. It is not settable as a field in the request body.
  *   `selfLink` (*type:* `String.t`, *default:* `nil`) - [Output Only] Server-defined URL for the resource.
  *   `status` (*type:* `String.t`, *default:* `nil`) - [Output Only] The status of the public delegated prefix, which can be one of following values: - `INITIALIZING` The public delegated prefix is being initialized and addresses cannot be created yet. - `READY_TO_ANNOUNCE` The public delegated prefix is a live migration prefix and is active. - `ANNOUNCED` The public delegated prefix is active. - `DELETING` The public delegated prefix is being deprovsioned. 
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :byoipApiVersion => String.t() | nil,
          :creationTimestamp => String.t() | nil,
          :description => String.t() | nil,
          :fingerprint => String.t() | nil,
          :id => String.t() | nil,
          :ipCidrRange => String.t() | nil,
          :isLiveMigration => boolean() | nil,
          :kind => String.t() | nil,
          :name => String.t() | nil,
          :parentPrefix => String.t() | nil,
          :publicDelegatedSubPrefixs =>
            list(GoogleApi.Compute.V1.Model.PublicDelegatedPrefixPublicDelegatedSubPrefix.t())
            | nil,
          :region => String.t() | nil,
          :selfLink => String.t() | nil,
          :status => String.t() | nil
        }

  field(:byoipApiVersion)
  field(:creationTimestamp)
  field(:description)
  field(:fingerprint)
  field(:id)
  field(:ipCidrRange)
  field(:isLiveMigration)
  field(:kind)
  field(:name)
  field(:parentPrefix)

  field(:publicDelegatedSubPrefixs,
    as: GoogleApi.Compute.V1.Model.PublicDelegatedPrefixPublicDelegatedSubPrefix,
    type: :list
  )

  field(:region)
  field(:selfLink)
  field(:status)
end

defimpl Poison.Decoder, for: GoogleApi.Compute.V1.Model.PublicDelegatedPrefix do
  def decode(value, options) do
    GoogleApi.Compute.V1.Model.PublicDelegatedPrefix.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.Compute.V1.Model.PublicDelegatedPrefix do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
