# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# NOTE: This file is auto generated by the elixir code generator program.
# Do not edit this file manually.

defmodule GoogleApi.Compute.V1.Model.ResourcePolicy do
  @moduledoc """
  Represents a Resource Policy resource. You can use resource policies to schedule actions for some Compute Engine resources. For example, you can use them to schedule persistent disk snapshots.

  ## Attributes

  *   `creationTimestamp` (*type:* `String.t`, *default:* `nil`) - [Output Only] Creation timestamp in RFC3339 text format.
  *   `description` (*type:* `String.t`, *default:* `nil`) - 
  *   `diskConsistencyGroupPolicy` (*type:* `GoogleApi.Compute.V1.Model.ResourcePolicyDiskConsistencyGroupPolicy.t`, *default:* `nil`) - Resource policy for disk consistency groups.
  *   `groupPlacementPolicy` (*type:* `GoogleApi.Compute.V1.Model.ResourcePolicyGroupPlacementPolicy.t`, *default:* `nil`) - Resource policy for instances for placement configuration.
  *   `id` (*type:* `String.t`, *default:* `nil`) - [Output Only] The unique identifier for the resource. This identifier is defined by the server.
  *   `instanceSchedulePolicy` (*type:* `GoogleApi.Compute.V1.Model.ResourcePolicyInstanceSchedulePolicy.t`, *default:* `nil`) - Resource policy for scheduling instance operations.
  *   `kind` (*type:* `String.t`, *default:* `compute#resourcePolicy`) - [Output Only] Type of the resource. Always compute#resource_policies for resource policies.
  *   `name` (*type:* `String.t`, *default:* `nil`) - The name of the resource, provided by the client when initially creating the resource. The resource name must be 1-63 characters long, and comply with RFC1035. Specifically, the name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character must be a lowercase letter, and all following characters must be a dash, lowercase letter, or digit, except the last character, which cannot be a dash.
  *   `region` (*type:* `String.t`, *default:* `nil`) - 
  *   `resourceStatus` (*type:* `GoogleApi.Compute.V1.Model.ResourcePolicyResourceStatus.t`, *default:* `nil`) - [Output Only] The system status of the resource policy.
  *   `selfLink` (*type:* `String.t`, *default:* `nil`) - [Output Only] Server-defined fully-qualified URL for this resource.
  *   `snapshotSchedulePolicy` (*type:* `GoogleApi.Compute.V1.Model.ResourcePolicySnapshotSchedulePolicy.t`, *default:* `nil`) - Resource policy for persistent disks for creating snapshots.
  *   `status` (*type:* `String.t`, *default:* `nil`) - [Output Only] The status of resource policy creation.
  """

  use GoogleApi.Gax.ModelBase

  @type t :: %__MODULE__{
          :creationTimestamp => String.t() | nil,
          :description => String.t() | nil,
          :diskConsistencyGroupPolicy =>
            GoogleApi.Compute.V1.Model.ResourcePolicyDiskConsistencyGroupPolicy.t() | nil,
          :groupPlacementPolicy =>
            GoogleApi.Compute.V1.Model.ResourcePolicyGroupPlacementPolicy.t() | nil,
          :id => String.t() | nil,
          :instanceSchedulePolicy =>
            GoogleApi.Compute.V1.Model.ResourcePolicyInstanceSchedulePolicy.t() | nil,
          :kind => String.t() | nil,
          :name => String.t() | nil,
          :region => String.t() | nil,
          :resourceStatus => GoogleApi.Compute.V1.Model.ResourcePolicyResourceStatus.t() | nil,
          :selfLink => String.t() | nil,
          :snapshotSchedulePolicy =>
            GoogleApi.Compute.V1.Model.ResourcePolicySnapshotSchedulePolicy.t() | nil,
          :status => String.t() | nil
        }

  field(:creationTimestamp)
  field(:description)

  field(:diskConsistencyGroupPolicy,
    as: GoogleApi.Compute.V1.Model.ResourcePolicyDiskConsistencyGroupPolicy
  )

  field(:groupPlacementPolicy, as: GoogleApi.Compute.V1.Model.ResourcePolicyGroupPlacementPolicy)
  field(:id)

  field(:instanceSchedulePolicy,
    as: GoogleApi.Compute.V1.Model.ResourcePolicyInstanceSchedulePolicy
  )

  field(:kind)
  field(:name)
  field(:region)
  field(:resourceStatus, as: GoogleApi.Compute.V1.Model.ResourcePolicyResourceStatus)
  field(:selfLink)

  field(:snapshotSchedulePolicy,
    as: GoogleApi.Compute.V1.Model.ResourcePolicySnapshotSchedulePolicy
  )

  field(:status)
end

defimpl Poison.Decoder, for: GoogleApi.Compute.V1.Model.ResourcePolicy do
  def decode(value, options) do
    GoogleApi.Compute.V1.Model.ResourcePolicy.decode(value, options)
  end
end

defimpl Poison.Encoder, for: GoogleApi.Compute.V1.Model.ResourcePolicy do
  def encode(value, options) do
    GoogleApi.Gax.ModelBase.encode(value, options)
  end
end
